#ifndef __IFS__
#define	__IFS__

#ifndef __MYPOINTS__
#include "mypoints.h"
#endif

#ifndef __LIST__
#include "list.h"
#endif
extern long	gmetritis;


//IFS self-affine model
template	<class Type,class aType,class DType>
class IFS_SA
{
	protected:
		int		N; //total points (extracted)
		
	public:
		Points<DType>	*d;//the contraction factors
		Points2D<Type>	*IP;
		
		IFS_SA();
		IFS_SA (Points2D<Type>	*list_of_points, Points<DType> *list_of_c_factors,
					int N_value);
		IFS_SA (FILE *stream,long,double,double);
		IFS_SA (Points<aType>	*signal);
		~IFS_SA();
		
		virtual Points2D<double>		*w_map(Points2D<double>	*list);
		int							write_to_file(FILE *stream,double,double);
		void						print(void);
		virtual Points2D<double>		*produce_points	(long);
		void						dequantize_contraction_factors_one_byte
											(double min,double max);
		void						quantize_contraction_factors_one_byte
											(double min,double max);
		void						dequantize_contraction_factors_one_byte2
											(double min,double max);
		void						quantize_contraction_factors_one_byte2
											(double min,double max);
};
//
typedef IFS_SA<int,BYTE,double>	SIMPLE_IFS_SA_MODEL;



//IFS piecewise Self affine model
template	<class Type,class aType,class CType,class DType>
class IFS_PSA
{
	protected:
		int		N; //total points (extracted)
		int		delta;
		int		psi;
		BYTE	kapa;	//psi=k*delta;
		BYTE	extra;	//Extra points we created for better interpolation
		
	public:
		Points<CType>	*C;//sections, intervals
		Points<DType>	*d;//the contraction factors
		Points2D<Type>	*IP;
		Points2D<Type>	*VP;
		
				IFS_PSA();
				IFS_PSA	(Points2D<Type>	*list_of_points, Points<DType> *list_of_c_factors,
						Points<CType>	*list_c,int N_value,
						int delta_value,int kapa_value,int extra_value);
				IFS_PSA	(FILE *stream,long,int,int,double,double);
				IFS_PSA(Points<aType>	*signal,
										long delta_value,
										long psi_value,
										long extra_value,
										CONTRACTION_FACTORS_CALCULATION_TYPE type);
							~IFS_PSA();
		int					current_interval(void);
		void				goto_next_interval(void);
		int					delta_value(void);
		void				set_delta_value(int	delta_value);
		BYTE				kapa_value(void);
		void				set_kapa_value(BYTE	kapa_value);
		BYTE				extra_value(void);
		void				set_extra_value(BYTE	extra_value);
		int					psi_value(void);
		virtual Points2D<double>		*w_map(Points2D<double>	*list);
		virtual Points2D<double>		*w_map2(Points2D<double>	*list);
		int					write_to_file(FILE *stream,double,double);
		void				print(void);
		virtual Points2D<double>		*produce_points	(BYTE iterations);
		void				dequantize_contraction_factors_one_byte
											(double min,double max);
		void				quantize_contraction_factors_one_byte
											(double min,double max);
		void				dequantize_contraction_factors_one_byte2
											(double min,double max);
		void				quantize_contraction_factors_one_byte2
											(double min,double max);
};
//this is for pictures with resolution up to 2048x2048 if psi=8
//or 4096x4096 if psi=16
typedef IFS_PSA<int,BYTE,BYTE,double>	SIMPLE_IFS_PSA_MODEL;

template	<class Type,class aType,class CType,class DType>
class IFS_PSA_v2:public IFS_PSA<Type,aType,CType,DType>
{
	public:
		IFS_PSA_v2(Points<aType>	*signal,
								long delta_value,
								long psi_value,
								long extra_value,
								CONTRACTION_FACTORS_CALCULATION_TYPE type);
		IFS_PSA_v2	(FILE *,long,int,int,double,double);
		//~IFS_PSA_v2();
		Points2D<double>		*w_map(Points2D<double>	*list);
		Points2D<double>		*produce_points	(BYTE iterations);
		
};
//this is for pictures with resolution up to 2048x2048 if delta=8
//or 1024x1024 if delta=4
typedef IFS_PSA_v2<int,BYTE,BYTE,double>	SIMPLE_IFS_PSA_MODEL_v2;






template <class Type1,class Type2>
void	copy_points2D(Points2D<Type2> *fromPoints2D,Points2D<Type1> *&toPoints2D);
template <class Type>
double	LINE_SNR_RATIO(Points<Type> *signal1,Points<Type> *signal2);

template <class Type1,class Type2>
double		heucledian_distance(Points<Type1> *big_list,
								Points2D<Type2> *small_list,
								long	left, long right);

template <class Type1>
double	calculate_number_n(Points<Type1>	*big_list,
							long	left,long right);

template <class Type1>
double	analytic_calculation_of_contraction_factor(Points<Type1>	*big_list,
											long	left_section_index,
											long	right_section_index,
											long	left_interval_index,
											long	right_interval_index);
							
Points<BYTE>	*normalise_signal(Points2D<double>	*signal,long	N);
Points2D<double>	*w_map_j_interval_to_i_section(Points<BYTE>	*plist,
												long	left_interval,
												long	right_interval,
												long	left_section,
												long	right_section,
												double	c_factor);
Points<BYTE> *Quantisation_of_contraction_factors(Points<double>	*d,
												double	min,
												double max);
Points<double> *DeQuantisation_of_contraction_factors(Points<BYTE>	*qd,
													double	min,
													double 	max);
Points<BYTE> *Quantisation_of_contraction_factors2(Points<double>	*d,
												double	min,
												double max);
Points<double> *DeQuantisation_of_contraction_factors2(Points<BYTE>	*qd,
													double	min,
													double 	max);

//***************************************************************
//IFS_SA related Functions
template	<class Type,class aType,class DType>
IFS_SA<Type,aType,DType>::IFS_SA()
{
	;
}

template	<class Type,class aType,class DType>
IFS_SA<Type,aType,DType>::IFS_SA (Points2D<Type>	*list_of_points, 
							Points<DType> *list_of_c_factors,int N_value)
{
	N=N_value;
	d=list_of_c_factors;
	IP=list_of_points;
}

template	<class Type,class aType,class DType>
IFS_SA<Type,aType,DType>::IFS_SA (FILE *stream,long Nv,double c_min,double c_max)
{
	int		items_red;
	N=Nv;
	long	number_of_interpolation_points;
	
	if (items_red=fread(&number_of_interpolation_points,sizeof(long),1,stream)!=1)
		{gerror.set(1);return;}
	
	Points<BYTE>	*qd=new Points<BYTE>(number_of_interpolation_points-1);
	if (gerror.error_occured())
		return;
	if (qd==NULL)
		{gerror.set(-1);return;}
	if (items_red=fread(qd->return_address(),sizeof(BYTE),qd->total_points(),stream)!=qd->total_points())
		{gerror.set(1);return;}
	qd->set_used(number_of_interpolation_points-1);
	d=DeQuantisation_of_contraction_factors2(qd,c_min,c_max);
	delete qd;
	IP=new Points2D<Type>(number_of_interpolation_points);
	if (gerror.error_occured())
		return;
	if (IP==NULL)
		{gerror.set(-1);return;}
	
	Points<Type>	*IPx;
	IPx=new Points<Type>(number_of_interpolation_points);
	if (gerror.error_occured())
		return;
	if (IPx==NULL)
		{gerror.set(-1);return;}
	if (items_red=fread(IPx->return_address(),sizeof(Type),IPx->total_points(),stream)!=IPx->total_points())
		{delete IPx;gerror.set(1);return;}
	IPx->set_used(IPx->total_points());
	
	Points<aType>  *IPy;
	IPy=new Points<aType>(number_of_interpolation_points);
	if (gerror.error_occured())
		return;
	if (IPy==NULL)
		{gerror.set(-1);return;}
	if (items_red=fread(IPy->return_address(),sizeof(aType),IPy->total_points(),stream)!=IPy->total_points())
		{delete IPy;gerror.set(1);return;}
	IPy->set_used(IPy->total_points());
	
	long	i;
	IP->goto_first_point();
	IPx->goto_first_point();	
	IPy->goto_first_point();	
	for (i=1;i<=IPx->total_points();i++)
	{
		IP->setx_value(IPx->point_value());
		IP->sety_value(IPy->point_value());
		IP->goto_next_point();
		IPx->goto_next_point();
		IPy->goto_next_point();
	}
	
	delete IPx;
	delete IPy;
	IP->goto_first_point();
	
	return;
}

template	<class Type,class aType,class DType>
IFS_SA<Type,aType,DType>::IFS_SA (Points<aType>	*signal)
{
	LIST<Type>		index_list;
	LIST<DType>		d_list;
	long			min_rightpoint;
	double			contraction_factor,min_distance,min_contraction_factor,temp_distance;
	long			leftpoint,rightpoint,M;
	Points2D<double>	*temp_points;
	Type*			item_pointer;
	DType*			item_pointer2;
	double			ni,mi;
	int				first_time;
	long			i;
	long			max_step,min_step;
	
	N=signal->total_points()-1;
	max_step=N/2;
	min_step=4;
	leftpoint=0;
	item_pointer=new Type;
	*item_pointer=0;
	index_list.put_item_to_end(item_pointer);
	ni=calculate_number_n(signal,1,N);
	while (leftpoint<N)
	{
		rightpoint=leftpoint+max_step;
		if (rightpoint>N)
			rightpoint=N;
		first_time=1;
		if (rightpoint<=leftpoint+min_step)
		{
			mi=calculate_number_n(signal,leftpoint,rightpoint);
			if (ni==0)
				contraction_factor=1;
			else
				contraction_factor=mi/ni;
			item_pointer=new Type;
			*item_pointer=rightpoint;
			index_list.put_item_to_end(item_pointer);
			item_pointer2=new DType;
			*item_pointer2=contraction_factor;
			d_list.put_item_to_end(item_pointer2);
			break;
		}
		while (rightpoint>leftpoint+min_step)
		{
			mi=calculate_number_n(signal,leftpoint,rightpoint);
			if (ni==0)
				contraction_factor=1;
			else
				contraction_factor=mi/ni;
			temp_points=w_map_j_interval_to_i_section(signal,1,
						N,leftpoint+1,rightpoint+1,
						contraction_factor);
			temp_distance=heucledian_distance(signal,temp_points,
					leftpoint+1,rightpoint+1);
			if (first_time==1)
			{
				first_time=0;
				min_distance=temp_distance;
				min_rightpoint=rightpoint;
				min_contraction_factor=contraction_factor;
			}
			else if (temp_distance<min_distance)
			{
				min_distance=temp_distance;
				min_rightpoint=rightpoint;
				min_contraction_factor=contraction_factor;
			}
			delete temp_points;
			rightpoint--;
		}
		item_pointer=new Type;
		*item_pointer=min_rightpoint;
		index_list.put_item_to_end(item_pointer);
		leftpoint=min_rightpoint;
		item_pointer2=new DType;
		*item_pointer2=min_contraction_factor;
		d_list.put_item_to_end(item_pointer2);
	}
	IP=new Points2D<Type>(index_list.total_items());
	d=new Points<DType>(d_list.total_items());
	
	Type	*xptr,y;
	index_list.goto_first_item();
	M=index_list.total_items();
	for (i=1;i<=M;i++)
	{
		xptr=index_list.return_current_item();
		signal->goto_index(*xptr);
		y=signal->point_value();
		IP->put_point(*xptr,y);
		index_list.goto_next_item();
	}
	
	DType	*d_valueptr;
	d_list.goto_first_item();
	M=d_list.total_items();
	for (i=1;i<=M;i++)
	{
		d_valueptr=d_list.return_current_item();
		d->put_point(*d_valueptr);
		d_list.goto_next_item();
	}
}



template	<class Type,class aType,class DType>
IFS_SA<Type,aType,DType>::~IFS_SA()
{	
	if (this!=NULL)
	{
		delete d;
		delete IP;
		#ifdef __DEBUG__
			cout << "IFS_SA deallocated" << "\n" ;
		#endif
	}
}


template	<class Type,class aType,class DType>
Points2D<double>	*IFS_SA<Type,aType,DType>::w_map(Points2D<double>	*list)
{
	Points2D<double>		*newlist=new Points2D<double>(d->total_points()*(list->total_points()-1)+1);
	if (gerror.error_occured())
		return(NULL);
	if (newlist==NULL)
		{gerror.set(-1);return(NULL);}
	int			number_of_sections,section;
	double		a,b,c,f,c_factor,x0,y0,xm,ym,xi1,yi1,xi,yi,wx,wy;
	int			section_index;
	
	IP->goto_first_point();
	list->goto_first_point();
	d->goto_first_point();
	number_of_sections=IP->total_points()-1;
	section_index=1;
	x0=IP->pointx_value();
	y0=IP->pointy_value();
	IP->goto_index(IP->total_points());
	xm=IP->pointx_value();
	ym=IP->pointy_value();
	IP->goto_first_point();
	for (section=1;section<=number_of_sections;section++)
	{
		//for every section do:
		//find the parameters of the map
		xi1=IP->pointx_value();
		yi1=IP->pointy_value();
		IP->goto_next_point();
		xi=IP->pointx_value();
		yi=IP->pointy_value();
		c_factor=d->point_value();
		d->goto_next_point();
		a=(xi-xi1)/(xm-x0);
		//c=xi-a*xm;
		c=(xm*xi1-x0*xi)/(xm-x0);
		//b=(yi-yi1)/(xm-x0)+((ym-y0)/(xm-x0))*c_factor;
		b=(yi-yi1)/(xm-x0)-c_factor*(ym-y0)/(xm-x0);
		//f=yi-b*xm-c_factor*ym;
		f=(xm*yi1-x0*yi)/(xm-x0)-c_factor*(xm*y0-x0*ym)/(xm-x0);
		
		/*cout<<"------------------\n"<<"section:"<<section<<"\n";
		cout<<"a="<<a<<"\n";
		cout<<"b="<<b<<"\n";
		cout<<"c="<<c<<"\n";
		cout<<"f="<<f<<"\n";*/
		list->goto_first_point();
		while (list->index_value()<list->total_points())
		{
			wx=a*list->pointx_value()+c;
			wy=b*list->pointx_value()+c_factor*list->pointy_value()+f;
			newlist->put_point(wx,wy);
			list->goto_next_point();
		}
	}
	//the last point is the same, so there is no need to
	//evaluate it again...
	newlist->put_point(list->pointx_value(),list->pointy_value());
	return(newlist);
}

template	<class Type,class aType,class DType>
int		IFS_SA<Type,aType,DType>::write_to_file (FILE *stream,
														double	c_min,
														double	c_max)
{
	int				response;
	Points<BYTE>	*qd;
	long			NUMBER;
	
	NUMBER=IP->total_points();
	response=fwrite(&NUMBER,sizeof(long),1,stream);
	if (response!=1)
		return(FALSE);
				
	qd=Quantisation_of_contraction_factors2(d,c_min,c_max);
	response=fwrite(qd->return_address(),sizeof(BYTE),qd->how_many_points(),stream);
	if (response!=qd->how_many_points())
		return(FALSE);
	Points<Type>	IPx(IP->total_points());
	Points<aType>	IPy(IP->total_points());
	long	i;
	IP->goto_first_point();
	for (i=1;i<=IP->total_points();i++)
	{
		IPx.put_point(IP->pointx_value());
		IPy.put_point(IP->pointy_value());
		IP->goto_next_point();
	}
	response=fwrite(IPx.return_address(),sizeof(Type),IPx.how_many_points(),stream);
	if (response!=IPx.how_many_points())
		return(FALSE);
	response=fwrite(IPy.return_address(),sizeof(aType),IPy.how_many_points(),stream);
	if (response!=IPy.how_many_points())
		return(FALSE);
	delete qd;
	return(TRUE);
}

template	<class Type,class aType,class DType>
void		IFS_SA<Type,aType,DType>::print(void)
{
	cout<< "\nPrinting IFS Model\n";
	cout<<"total interpolation points:"<<N<<"\n";
	d->print_points();
	cout<< "Interpolation points\n";
	IP->print_points();
}

template	<class Type,class aType,class DType>
Points2D<double>		*IFS_SA<Type,aType,DType>::produce_points(long	number_of_points)
{
	Points2D<double>	*new_points,*old_points;
	BYTE	i;
	int				iterations;
	
	iterations=25;
	
	copy_points2D(IP,old_points);
	//cout<<"old:"<<old_points->return_address()<<"\n";
	//cout<<"ifs_model:"<<IP->return_address()<<"\n";
	new_points=w_map(old_points);
	if (gerror.error_occured())
		return(NULL);
	#ifdef __DEBUG__
	//cout<<"iteration:"<<1<<"\n";
	//new_points->print_points();
	#endif
	for(i=2;i<=iterations;i++)
	{
		if (new_points->total_points()>=2*number_of_points)
			break;
		old_points=new_points;
		new_points=w_map(old_points);
		if (gerror.error_occured())
			return(NULL);
		delete old_points;
	}
	return(new_points);
}

template	<class Type,class aType,class DType>
void		IFS_SA<Type,aType,DType>::quantize_contraction_factors_one_byte(
												double min,double max)
{
	double	maxmin;
	long	i;
	
	d->goto_first_point();
	if (max!=min)
		maxmin=max-min;
	else
	{
		max=min+1;
		maxmin=1;
	}
	for(i=1;i<=d->total_points();i++)
	{
		d->set_point(round(255*(d->point_value()-min)/(maxmin)));
		d->goto_next_point();
	}
}

template	<class Type,class aType,class DType>
void		IFS_SA<Type,aType,DType>::dequantize_contraction_factors_one_byte(
												double min,double max)
{
	double	maxmin;
	
	long	i;
	d->goto_first_point();
	if (max!=min)
		maxmin=max-min;
	else
	{
		max=min+1;
		maxmin=1;
	}
	for(i=1;i<=d->total_points();i++)
	{
		d->set_point(d->point_value()*maxmin/255+min);
		d->goto_next_point();
	}
}

template	<class Type,class aType,class DType>
void		IFS_SA<Type,aType,DType>::quantize_contraction_factors_one_byte2(
												double min,double max)
{
	double	maxmin;
	long	i;
	double	MAX_VALUE=180;
	
	d->goto_first_point();
	
	for(i=1;i<=d->total_points();i++)
	{
		if (d->point_value()<=1)
			d->set_point(round((d->point_value()-min)*MAX_VALUE/(1-min)));
		else
			d->set_point(round((d->point_value()-1)*(255-MAX_VALUE)/(max-1)+MAX_VALUE));
		d->goto_next_point();
	}
}

template	<class Type,class aType,class DType>
void		IFS_SA<Type,aType,DType>::dequantize_contraction_factors_one_byte2(
												double min,double max)
{
	double	maxmin;
	long	i;
	double	MAX_VALUE=180;
	d->goto_first_point();
	for(i=1;i<=d->total_points();i++)
	{
		if (d->point_value()<=MAX_VALUE)
			d->set_point(d->point_value()*(1-min)/MAX_VALUE+min);
		else
			d->set_point((d->point_value()-MAX_VALUE)*(max-1)/(255-MAX_VALUE)+1);
		d->goto_next_point();
	}
}



//***************************************************************
//IFS_PSA related Functions
template	<class Type,class aType,class CType,class DType>
IFS_PSA<Type,aType,CType,DType>::IFS_PSA()
{
	;
}

template	<class Type,class aType,class CType,class DType>
IFS_PSA<Type,aType,CType,DType>::IFS_PSA	(Points2D<Type>	*list_of_points, Points<DType> *list_of_c_factors,
				Points<CType>	*list_c,int N_value,
				int delta_value,int kapa_value,int extra_value)
{
	long		i,j;
	
	delta=delta_value;
	kapa=kapa_value;
	psi=kapa*delta;
	extra=extra_value;
	IP=list_of_points;
	N=N_value;
	VP=new Points2D<Type>(N/psi+1);
	if (gerror.error_occured())
		return;
	if (VP==NULL)
		{gerror.set(-1);return(NULL);}
	IP->goto_first_point();
	for (i=0;i<=VP->total_points();i++)
	{
		VP->put_point(psi*i,IP->pointy_value());
		IP->move_i_points(kapa);
	}
	d=list_of_c_factors;
	C=list_c;
	#ifdef __DEBUG__
		cout << "IFS_PSA allocated" << "\n" ;
	#endif
}

template	<class Type,class aType,class CType,class DType>
IFS_PSA<Type,aType,CType,DType>::IFS_PSA(FILE	*stream,long Nv,int deltav,int psiv,
										double c_min,double c_max)
{
	unsigned int		items_red;
	
	delta=deltav;
	psi=psiv;
	kapa=psi/delta;
	
	C=new Points<CType>((Nv-1)/deltav);
	if (gerror.error_occured())
		return;
	if (C==NULL)
		{gerror.set(-1);return;}
	if (items_red=fread(C->return_address(),sizeof(CType),C->total_points(),stream)!=C->total_points())
		{gerror.set(1);return;}
	C->set_used((Nv-1)/deltav);
	
	Points<BYTE>	*qd=new Points<BYTE>(C->total_points());
	if (gerror.error_occured())
		return;
	if (qd==NULL)
		{gerror.set(-1);return;}
	if (items_red=fread(qd->return_address(),sizeof(BYTE),qd->total_points(),stream)!=qd->total_points())
		{gerror.set(1);return;}
	qd->set_used((Nv-1)/deltav);
	d=DeQuantisation_of_contraction_factors2(qd,c_min,c_max);
	delete qd;
	IP=new Points2D<Type>(C->total_points()+1);
	if (gerror.error_occured())
		return;
	if (IP==NULL)
		{gerror.set(-1);return;}
	VP=new Points2D<Type>( ((Nv-1)/psiv)+1);
	if (gerror.error_occured())
		return;
	if (VP==NULL)
		{gerror.set(-1);return;}
	Points<aType>  *IP2;
	IP2=new Points<aType>(C->total_points()+1);
	if (gerror.error_occured())
		return;
	if (IP2==NULL)
		{gerror.set(-1);return;}
	if (items_red=fread(IP2->return_address(),sizeof(aType),IP2->total_points(),stream)!=IP2->total_points())
		{delete IP2;gerror.set(1);return;}
	IP2->set_used(IP2->total_points());
	long	i,index=0;
	
	IP->goto_first_point();
	IP2->goto_first_point();	
	for (i=1;i<=IP2->total_points();i++)
	{
		IP->setx_value(index);
		IP->sety_value(IP2->point_value());
		IP->goto_next_point();
		IP2->goto_next_point();
		index=index+deltav;
	}
	IP->goto_first_point();
	VP->goto_first_point();
	index=0;	
	for (i=1;i<=VP->total_points();i++)
	{
		VP->setx_value(index);
		VP->sety_value(IP->pointy_value());
		VP->goto_next_point();
		IP->move_i_points(kapa);
		index=index+psiv;
	}
	delete IP2;
	IP->goto_first_point();
	
	return;
}

template	<class Type,class aType,class CType,class DType>
IFS_PSA<Type,aType,CType,DType>::IFS_PSA(Points<aType>	*signal,
										long delta_value,
										long psi_value,
										long extra_value,
										CONTRACTION_FACTORS_CALCULATION_TYPE type)
{
	N=signal->total_points();
	delta=delta_value;
	psi=psi_value;
	kapa=psi/delta;
	extra=extra_value;
	
	long	start_signal_index=signal->index_value();
	long	M=(signal->total_points()-1)/delta;//number of sections
	long	M1=(signal->total_points()-1)/psi;//number of intervals
	long	i,j,left_section_index=1,left_interval_index=1;
	long	right_section_index=1+delta,right_interval_index=1+psi;
	char	first_time;
	double	ni,mi,contraction_factor_i_j;
	Points2D<double>	*temp_points;
	double	temp_distance,min_distance,min_contraction_factor;
	long	min_j;
	
	C=new Points<CType>(M);
	if (gerror.error_occured())
		return;
	if (C==NULL)
		{gerror.set(-1);return;}
	d=new Points<DType>(M);
	if (gerror.error_occured())
		return;
	if (d==NULL)
		{gerror.set(-1);return;}
	IP=new Points2D<Type>(M+1);
	if (gerror.error_occured())
		return;
	if (IP==NULL)
		{gerror.set(-1);return;}
	VP=new Points2D<Type>(M1+1);
	if (gerror.error_occured())
		return;
	if (VP==NULL)
		{gerror.set(-1);return;}
	
	signal->goto_index(left_section_index);
	for (i=1;i<=M+1;i++)//Construct IP
	{
		IP->put_point(left_section_index-1,signal->point_value());
		left_section_index+=delta;
		signal->goto_index(left_section_index);
	}
	signal->goto_index(left_interval_index);
	for (j=1;j<=M1+1;j++)//Construct VP
	{
		VP->put_point(left_interval_index-1,signal->point_value());
		left_interval_index+=psi;
		signal->goto_index(left_interval_index);
	}
	
	left_section_index=1;
	right_section_index=1+delta;
	left_interval_index=1;
	right_interval_index=1+psi;
	for (i=1;i<=M;i++)//for every section
	{
		first_time=1;
		left_interval_index=1;
		right_interval_index=1+psi;
		for (j=1;j<=M1;j++)//for every interval
		{
			//interval distance
			#ifdef __DEBUG__
			cout<<"------------------\n";
			cout<<"section:"<<i<<" Interval:"<<j<<"\n";
			cout<<"left_interval_index: "<<left_interval_index<<"\n";
			cout<<"right_interval_index: "<<right_interval_index<<"\n";
			cout<<"left_section_index: "<<left_section_index<<"\n";
			cout<<"right_section_index: "<<right_section_index<<"\n";
			#endif
			if (type==geometric)
			{
				ni=calculate_number_n(signal,left_interval_index,right_interval_index);
				mi=calculate_number_n(signal,left_section_index,right_section_index);
				if (ni==0)
					contraction_factor_i_j=1;
				else
					contraction_factor_i_j=mi/ni;
			}
			else
				contraction_factor_i_j=analytic_calculation_of_contraction_factor(signal,left_section_index,right_section_index,left_interval_index,right_interval_index);
			
			temp_points=w_map_j_interval_to_i_section(signal,left_interval_index,
					right_interval_index,left_section_index,right_section_index,
					contraction_factor_i_j);
			if (gerror.error_occured())
				return;
			#ifdef __DEBUG__
			temp_points->print_points();
			#endif
			temp_distance=heucledian_distance(signal,temp_points,
					left_section_index,right_section_index);
			if (first_time==1)
			{
				first_time=0;
				min_distance=temp_distance;
				min_j=j;
				min_contraction_factor=contraction_factor_i_j;
			}
			else if (temp_distance<min_distance)
			{
				min_distance=temp_distance;
				min_j=j;
				min_contraction_factor=contraction_factor_i_j;
			}
			delete temp_points;
			left_interval_index=right_interval_index;
			right_interval_index+=psi;
		}
		d->put_point(min_contraction_factor);
		/*if (min_contraction_factor>1)
			gmetritis++;*/
		C->put_point(min_j);
		left_section_index=right_section_index;
		right_section_index+=delta;
	}
	signal->goto_index(start_signal_index+1);
}


template	<class Type,class aType,class CType,class DType>
IFS_PSA<Type,aType,CType,DType>::~IFS_PSA()
{	
	if (this!=NULL)
	{
		delete C;
		delete d;
		delete IP;
		delete VP;
		#ifdef __DEBUG__
			cout << "IFS_PSA deallocated" << "\n" ;
		#endif
	}
}

template	<class Type,class aType,class CType,class DType>
int		IFS_PSA<Type,aType,CType,DType>::current_interval(void)
{
	return(VP->index_value());
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::goto_next_interval(void)
{
	VP->goto_next_point();
}

template	<class Type,class aType,class CType,class DType>
int		IFS_PSA<Type,aType,CType,DType>::delta_value(void)
{
	return(delta);
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::set_delta_value(int	delta_value)
{
	delta=delta_value;
}

template	<class Type,class aType,class CType,class DType>
BYTE		IFS_PSA<Type,aType,CType,DType>::kapa_value(void)
{
	return(kapa);
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::set_kapa_value(BYTE	kapa_value)
{
	kapa=kapa_value;
}

template	<class Type,class aType,class CType,class DType>
BYTE		IFS_PSA<Type,aType,CType,DType>::extra_value(void)
{
	return(extra);
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::set_extra_value(BYTE	extra_value)
{
	extra=extra_value;
}

template	<class Type,class aType,class CType,class DType>
int		IFS_PSA<Type,aType,CType,DType>::psi_value(void)
{
	return(psi);
}

template	<class Type,class aType,class CType,class DType>
Points2D<double>	*IFS_PSA<Type,aType,CType,DType>::w_map(Points2D<double>	*list)
{
	Points2D<double>		*newlist=new Points2D<double>(kapa*(list->total_points()-1)+1);
	if (gerror.error_occured())
		return(NULL);
	if (newlist==NULL)
		{gerror.set(-1);return(NULL);}
	int			number_of_sections,section,interval;
	double		a,b,c,f,c_factor,x0,y0,xm,ym,xi1,yi1,xi,yi,wx,wy;
	int			section_index,interval_index,i;
	
	IP->goto_first_point();
	list->goto_first_point();
	C->goto_first_point();
	d->goto_first_point();
	number_of_sections=IP->total_points()-1;
	section_index=1;
	for (section=1;section<=number_of_sections;section++)
	{
		//for every section do:
		interval=C->point_value();//find the corresponding interval
		C->goto_next_point();
		VP->goto_index(interval);
		interval_index=list->findx_value_index(VP->pointx_value());
		//find the parameters of the map
		x0=VP->pointx_value();
		y0=VP->pointy_value();
		VP->goto_next_point();
		xm=VP->pointx_value();
		ym=VP->pointy_value();
		xi1=IP->pointx_value();
		yi1=IP->pointy_value();
		IP->goto_next_point();
		xi=IP->pointx_value();
		yi=IP->pointy_value();
		c_factor=d->point_value();
		d->goto_next_point();
		a=(xi-xi1)/(xm-x0);
		//c=xi-a*xm;
		c=(xm*xi1-x0*xi)/(xm-x0);
		//b=(yi-yi1)/(xm-x0)+((ym-y0)/(xm-x0))*c_factor;
		b=(yi-yi1)/(xm-x0)-c_factor*(ym-y0)/(xm-x0);
		//f=yi-b*xm-c_factor*ym;
		f=(xm*yi1-x0*yi)/(xm-x0)-c_factor*(xm*y0-x0*ym)/(xm-x0);
		
		/*cout<<"------------------\n"<<"section:"<<section<<"\n";
		cout<<"interval:"<<interval<<"\n";
		cout<<"a="<<a<<"\n";
		cout<<"b="<<b<<"\n";
		cout<<"c="<<c<<"\n";
		cout<<"f="<<f<<"\n";*/
		list->goto_index(interval_index);
		while (list->pointx_value()!=xm)
		{
			#ifdef __DEBUG__
			//cout<<"point:"<<list->index_value()<<"\n";
			#endif
			//the first point is the same, so there is no need to
			//evaluate it again...
			list->goto_index(section_index);
			newlist->put_point(list->pointx_value(),list->pointy_value());
			list->goto_index(++interval_index);
			//evaluate only k-1 points every time 
			//the other are the same
			for (i=1;i<=kapa-1;i++)
			{
				wx=a*list->pointx_value()+c;
				wy=b*list->pointx_value()+c_factor*list->pointy_value()+f;
				newlist->put_point(wx,wy);
				list->goto_next_point();
			}
			interval_index+=kapa-1;
			++section_index;
		}
	}
	//the last point is the same, so there is no need to
	//evaluate it again...
	list->goto_index(section_index);
	newlist->put_point(list->pointx_value(),list->pointy_value());
	return(newlist);
}

template	<class Type,class aType,class CType,class DType>
Points2D<double>	*IFS_PSA<Type,aType,CType,DType>::w_map2(Points2D<double>	*list)
{
	Points2D<double>		*newlist=new Points2D<double>(kapa*(list->total_points()-1)+1);
	if (gerror.error_occured())
		return(NULL);
	if (newlist==NULL)
		{gerror.set(-1);return(NULL);}
	int			number_of_sections,section,interval;
	double		a,b,c,f,c_factor,x0,y0,xm,ym,xi1,yi1,xi,yi,wx,wy;
	int			interval_index;
	
	IP->goto_first_point();
	list->goto_first_point();
	C->goto_first_point();
	d->goto_first_point();
	number_of_sections=IP->total_points()-1;
	for (section=1;section<=number_of_sections;section++)
	{
		//for every section do:
		interval=C->point_value();//find the corresponding interval
		C->goto_next_point();
		VP->goto_index(interval);
		interval_index=list->findx_value_index(VP->pointx_value());
		//find the parameters of the map
		x0=VP->pointx_value();
		y0=VP->pointy_value();
		VP->goto_next_point();
		xm=VP->pointx_value();
		ym=VP->pointy_value();
		xi1=IP->pointx_value();
		yi1=IP->pointy_value();
		IP->goto_next_point();
		xi=IP->pointx_value();
		yi=IP->pointy_value();
		c_factor=d->point_value();
		d->goto_next_point();
		a=(xi-xi1)/(xm-x0);
		//c=xi-a*xm;
		c=(xm*xi1-x0*xi)/(xm-x0);
		//b=(yi-yi1)/(xm-x0)+((ym-y0)/(xm-x0))*c_factor;
		b=(yi-yi1)/(xm-x0)-c_factor*(ym-y0)/(xm-x0);
		//f=yi-b*xm-c_factor*ym;
		f=(xm*yi1-x0*yi)/(xm-x0)-c_factor*(xm*y0-x0*ym)/(xm-x0);
		
		/*
		cout<<"------------------\n"<<"section:"<<section<<"\n";
		cout<<"interval:"<<interval<<"\n";
		cout<<"Xo="<<x0<<"\n";
		cout<<"Yo="<<y0<<"\n";
		cout<<"Xm="<<xm<<"\n";
		cout<<"Ym="<<ym<<"\n";
		cout<<"Xi-1="<<xi1<<"\n";
		cout<<"Yi-1="<<yi1<<"\n";
		cout<<"Xi="<<xi<<"\n";
		cout<<"Yi="<<yi<<"\n";
		cout<<"a="<<a<<"\n";
		cout<<"b="<<b<<"\n";
		cout<<"c="<<c<<"\n";
		cout<<"f="<<f<<"\n";
		cout<<"c_factor="<<c_factor<<"\n";*/
		list->goto_index(interval_index);
		while (list->pointx_value()!=xm)
		{
			wx=a*list->pointx_value()+c;
			wy=b*list->pointx_value()+c_factor*list->pointy_value()+f;
			newlist->put_point(wx,wy);
			list->goto_next_point();
		}
	}
	//the last point is the same, so there is no need to
	//evaluate it again...
	newlist->put_point(list->pointx_value(),list->pointy_value());
	return(newlist);
}

template	<class Type,class aType,class CType,class DType>
int		IFS_PSA<Type,aType,CType,DType>::write_to_file (FILE *stream,
														double	c_min,
														double	c_max)
{
	unsigned int	response;
	Points<BYTE>	*qd;
	
	response=fwrite(C->return_address(),sizeof(CType),C->how_many_points(),stream);
	if (response!=C->how_many_points())
		return(FALSE);
		
	qd=Quantisation_of_contraction_factors2(d,c_min,c_max);
	response=fwrite(qd->return_address(),sizeof(BYTE),qd->how_many_points(),stream);
	if (response!=qd->how_many_points())
		return(FALSE);
	Points<aType>	IP2(IP->total_points());
	long	i;
	IP->goto_first_point();
	for (i=1;i<=IP->total_points();i++)
	{
		IP2.put_point(IP->pointy_value());
		IP->goto_next_point();
	}
	response=fwrite(IP2.return_address(),sizeof(aType),IP2.how_many_points(),stream);
	if (response!=IP2.how_many_points())
		return(FALSE);
	delete qd;
	return(TRUE);
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::print(void)
{
	cout<< "\nPrinting IFS Model\n";
	cout<<"total interpolation points:"<<N<<"\n";
	cout<<"delta="<<delta<<"\npsi="<<psi<<"\nextra="<<extra<<"\n";
	C->print_points();
	d->print_points();
	cout<< "Interpolation points\n";
	IP->print_points();
	VP->print_points();
}

template	<class Type,class aType,class CType,class DType>
Points2D<double>		*IFS_PSA<Type,aType,CType,DType>::produce_points(BYTE	iterations)
{
	Points2D<double>	*new_points,*old_points;
	BYTE	i;
	
	copy_points2D(IP,old_points);
	//cout<<"old:"<<old_points->return_address()<<"\n";
	//cout<<"ifs_model:"<<IP->return_address()<<"\n";
	new_points=w_map(old_points);
	if (gerror.error_occured())
		return(NULL);
	#ifdef __DEBUG__
	//cout<<"iteration:"<<1<<"\n";
	//new_points->print_points();
	#endif
	for(i=2;i<=iterations;i++)
	{
		old_points=new_points;
		new_points=w_map(old_points);
		if (gerror.error_occured())
			return(NULL);
		#ifdef __DEBUG__
		//cout<<"iteration:"<<i<<"\n";
		//new_points->print_points();
		#endif
		delete old_points;
	}
	return(new_points);
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::quantize_contraction_factors_one_byte(
												double min,double max)
{
	double	maxmin;
	long	i;
	
	d->goto_first_point();
	if (max!=min)
		maxmin=max-min;
	else
	{
		max=min+1;
		maxmin=1;
	}
	for(i=1;i<=d->total_points();i++)
	{
		d->set_point(round(255*(d->point_value()-min)/(maxmin)));
		d->goto_next_point();
	}
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::dequantize_contraction_factors_one_byte(
												double min,double max)
{
	double	maxmin;
	
	long	i;
	d->goto_first_point();
	if (max!=min)
		maxmin=max-min;
	else
	{
		max=min+1;
		maxmin=1;
	}
	for(i=1;i<=d->total_points();i++)
	{
		d->set_point(d->point_value()*maxmin/255+min);
		d->goto_next_point();
	}
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::quantize_contraction_factors_one_byte2(
												double min,double max)
{
	long	i;
	double	MAX_VALUE=180;
	
	d->goto_first_point();
	
	for(i=1;i<=d->total_points();i++)
	{
		if (d->point_value()<=1)
			d->set_point(round((d->point_value()-min)*MAX_VALUE/(1-min)));
		else
			d->set_point(round((d->point_value()-1)*(255-MAX_VALUE)/(max-1)+MAX_VALUE));
		d->goto_next_point();
	}
}

template	<class Type,class aType,class CType,class DType>
void		IFS_PSA<Type,aType,CType,DType>::dequantize_contraction_factors_one_byte2(
												double min,double max)
{
	long	i;
	double	MAX_VALUE=180;
	d->goto_first_point();
	for(i=1;i<=d->total_points();i++)
	{
		if (d->point_value()<=MAX_VALUE)
			d->set_point(d->point_value()*(1-min)/MAX_VALUE+min);
		else
			d->set_point((d->point_value()-MAX_VALUE)*(max-1)/(255-MAX_VALUE)+1);
		d->goto_next_point();
	}
}


//***************************************************************
//IFS_PSA_v2 related Functions
template	<class Type,class aType,class CType,class DType>
IFS_PSA_v2<Type,aType,CType,DType>::IFS_PSA_v2(Points<aType>	*signal,
										long delta_value,
										long psi_value,
										long extra_value,
										CONTRACTION_FACTORS_CALCULATION_TYPE type):IFS_PSA<Type,aType,CType,DType>()
{
	N=signal->total_points();
	delta=delta_value;
	psi=psi_value;
	kapa=psi/delta;
	extra=extra_value;
	
	long	start_signal_index=signal->index_value();
	long	M=(signal->total_points()-1)/delta;//number of sections
	long	M1=(signal->total_points()-1)/psi;//number of intervals
	long	i,j,left_section_index=1,left_interval_index=1;
	long	right_section_index=1+delta,right_interval_index=1+psi;
	char	first_time;
	double	ni,mi,contraction_factor_i_j;
	Points2D<double>	*temp_points;
	double	temp_distance,min_distance,min_contraction_factor;
	long	min_j;
	
	C=new Points<CType>(M);
	if (gerror.error_occured())
		return;
	if (C==NULL)
		{gerror.set(-1);return;}
	d=new Points<DType>(M);
	if (gerror.error_occured())
		return;
	if (d==NULL)
		{gerror.set(-1);return;}
	IP=new Points2D<Type>(M+1);
	if (gerror.error_occured())
		return;
	if (IP==NULL)
		{gerror.set(-1);return;}
	VP=new Points2D<Type>(M1+1);
	if (gerror.error_occured())
		return;
	if (VP==NULL)
		{gerror.set(-1);return;}
	
	signal->goto_index(left_section_index);
	for (i=1;i<=M+1;i++)//Construct IP
	{
		IP->put_point(left_section_index-1,signal->point_value());
		left_section_index+=delta;
		signal->goto_index(left_section_index);
	}
	signal->goto_index(left_interval_index);
	for (j=1;j<=M1+1;j++)//Construct VP
	{
		VP->put_point(left_interval_index-1,signal->point_value());
		left_interval_index+=psi;
		signal->goto_index(left_interval_index);
	}
	
	left_section_index=1;
	right_section_index=1+delta;
	for (i=1;i<=M;i++)//for every section
	{
		first_time=1;
		left_interval_index=1;
		right_interval_index=1+psi;
		for (j=1;j<=M-kapa;j++)//for every interval
		{
			//interval distance
			#ifdef __DEBUG__
			cout<<"------------------\n";
			cout<<"section:"<<i<<" Interval:"<<j<<"\n";
			cout<<"left_interval_index: "<<left_interval_index<<"\n";
			cout<<"right_interval_index: "<<right_interval_index<<"\n";
			cout<<"left_section_index: "<<left_section_index<<"\n";
			cout<<"right_section_index: "<<right_section_index<<"\n";
			#endif
			
			if (type==geometric)
			{
				ni=calculate_number_n(signal,left_interval_index,right_interval_index);
				mi=calculate_number_n(signal,left_section_index,right_section_index);
				if (ni==0)
					contraction_factor_i_j=1;
				else
					contraction_factor_i_j=mi/ni;
			}
			else
				contraction_factor_i_j=analytic_calculation_of_contraction_factor(signal,left_section_index,right_section_index,left_interval_index,right_interval_index);
			
			temp_points=w_map_j_interval_to_i_section(signal,left_interval_index,
					right_interval_index,left_section_index,right_section_index,
					contraction_factor_i_j);


			temp_points=w_map_j_interval_to_i_section(signal,left_interval_index,
					right_interval_index,left_section_index,right_section_index,
					contraction_factor_i_j);
			if (gerror.error_occured())
				return;
			#ifdef __DEBUG__
			temp_points->print_points();
			#endif
			temp_distance=heucledian_distance(signal,temp_points,
					left_section_index,right_section_index);
			if (first_time==1)
			{
				first_time=0;
				min_distance=temp_distance;
				min_j=j;
				min_contraction_factor=contraction_factor_i_j;
			}
			else if (temp_distance<min_distance)
			{
				min_distance=temp_distance;
				min_j=j;
				min_contraction_factor=contraction_factor_i_j;
			}
			delete temp_points;
			left_interval_index+=delta;
			right_interval_index+=delta;
		}
		d->put_point(min_contraction_factor);
		C->put_point(min_j);
		left_section_index=right_section_index;
		right_section_index+=delta;
	}
	signal->goto_index(start_signal_index+1);
}



//******************************************
template	<class Type,class aType,class CType,class DType>
IFS_PSA_v2<Type,aType,CType,DType>::IFS_PSA_v2(FILE	*stream,long Nv,int deltav,int psiv,
												double c_min,double c_max)
:IFS_PSA<Type,aType,CType,DType>(stream,Nv,deltav,psiv,c_min,c_max)
{
}

//******************************************
template	<class Type,class aType,class CType,class DType>
Points2D<double>	*IFS_PSA_v2<Type,aType,CType,DType>::w_map(Points2D<double>	*list)
{
	Points2D<double>		*newlist=new Points2D<double>(kapa*(list->total_points()-1)+1);
	if (gerror.error_occured())
		return(NULL);
	if (newlist==NULL)
		{gerror.set(-1);return(NULL);}
	int			number_of_sections,section,interval;
	double		a,b,c,f,c_factor,x0,y0,xm,ym,xi1,yi1,xi,yi,wx,wy;
	int			section_index,interval_index,i,temp_index;
	
	IP->goto_first_point();
	list->goto_first_point();
	C->goto_first_point();
	d->goto_first_point();
	number_of_sections=IP->total_points()-1;
	section_index=1;
	for (section=1;section<=number_of_sections;section++)
	{
		//for every section do:
		interval=C->point_value();//find the corresponding interval
		C->goto_next_point();
		temp_index=IP->index_value();
		IP->goto_index(interval);
		interval_index=list->findx_value_index(IP->pointx_value());
		//find the parameters of the map
		x0=IP->pointx_value();
		y0=IP->pointy_value();
		for(i=1;i<=kapa;i++)
			IP->goto_next_point();
		xm=IP->pointx_value();
		ym=IP->pointy_value();
		IP->goto_index(temp_index);
		xi1=IP->pointx_value();
		yi1=IP->pointy_value();
		IP->goto_next_point();
		xi=IP->pointx_value();
		yi=IP->pointy_value();
		c_factor=d->point_value();
		d->goto_next_point();
		a=(xi-xi1)/(xm-x0);
		//c=xi-a*xm;
		c=(xm*xi1-x0*xi)/(xm-x0);
		//b=(yi-yi1)/(xm-x0)+((ym-y0)/(xm-x0))*c_factor;
		b=(yi-yi1)/(xm-x0)-c_factor*(ym-y0)/(xm-x0);
		//f=yi-b*xm-c_factor*ym;
		f=(xm*yi1-x0*yi)/(xm-x0)-c_factor*(xm*y0-x0*ym)/(xm-x0);
		
		/*cout<<"------------------\n"<<"section:"<<section<<"\n";
		cout<<"interval:"<<interval<<"\n";
		cout<<"a="<<a<<"\n";
		cout<<"b="<<b<<"\n";
		cout<<"c="<<c<<"\n";
		cout<<"f="<<f<<"\n";*/
		list->goto_index(interval_index);
		while (list->pointx_value()!=xm)
		{
			#ifdef __DEBUG__
			//cout<<"point:"<<list->index_value()<<"\n";
			#endif
			//the first point is the same, so there is no need to
			//evaluate it again...
			list->goto_index(section_index);
			newlist->put_point(list->pointx_value(),list->pointy_value());
			list->goto_index(++interval_index);
			//evaluate only k-1 points every time 
			//the other are the same
			for (i=1;i<=kapa-1;i++)
			{
				wx=a*list->pointx_value()+c;
				wy=b*list->pointx_value()+c_factor*list->pointy_value()+f;
				newlist->put_point(wx,wy);
				list->goto_next_point();
			}
			interval_index+=kapa-1;
			++section_index;
		}
	}
	//the last point is the same, so there is no need to
	//evaluate it again...
	list->goto_index(section_index);
	newlist->put_point(list->pointx_value(),list->pointy_value());
	return(newlist);
}

//*************************************
template	<class Type,class aType,class CType,class DType>
Points2D<double>		*IFS_PSA_v2<Type,aType,CType,DType>::produce_points(BYTE	iterations)
{
	Points2D<double>	*new_points,*old_points;
	BYTE	i;
	
	copy_points2D(IP,old_points);
	//cout<<"old:"<<old_points->return_address()<<"\n";
	//cout<<"ifs_model:"<<IP->return_address()<<"\n";
	new_points=w_map(old_points);
	if (gerror.error_occured())
		return(NULL);
	#ifdef __DEBUG__
	//cout<<"iteration:"<<1<<"\n";
	//new_points->print_points();
	#endif
	for(i=2;i<=iterations;i++)
	{
		old_points=new_points;
		new_points=w_map(old_points);
		if (gerror.error_occured())
			return(NULL);
		#ifdef __DEBUG__
		//cout<<"iteration:"<<i<<"\n";
		//new_points->print_points();
		#endif
		delete old_points;
	}
	return(new_points);
}



//********************************************************
//Independent functions
template <class Type1,class Type2>
void	copy_points2D(Points2D<Type2> *fromPoints2D,Points2D<Type1> *&toPoints2D)
{
	long				i,findex;
	
	toPoints2D=new Points2D<Type1>(fromPoints2D->total_points());
	if (gerror.error_occured())
		return;
	if (toPoints2D==NULL)
		{gerror.set(-1);return;}
	findex=fromPoints2D->index_value();
	fromPoints2D->goto_first_point();
	for (i=1;i<=fromPoints2D->total_points();i++)
	{
		toPoints2D->put_point(fromPoints2D->pointx_value(),fromPoints2D->pointy_value());
		fromPoints2D->goto_next_point();
	}
	fromPoints2D->goto_index(findex);
}


//it helps for the calculation of the contraction factor
template <class Type1>
double	calculate_number_n(Points<Type1>	*big_list,
							long	left,long right)
{
	long	big_list_index=big_list->index_value(),index;
	double	x0,y0,x1,y1,x,y,apost,max,lamda;
	
	big_list->goto_index(left);
	x0=big_list->index_value();
	y0=big_list->point_value();
	big_list->goto_index(right);
	x1=big_list->index_value();
	y1=big_list->point_value();
	big_list->goto_index(left+1);
	max=0;
	lamda=(y1-y0)/(x1-x0);
	for (index=left+1;index<=right-1;index++)
	{
		x=big_list->index_value();
		y=big_list->point_value();
		apost=fabs(y-lamda*(x-x0)-y0);
		if (apost>max)
			max=apost;
		big_list->goto_next_point();
	}
	return(max);
}

template <class Type1>
double	analytic_calculation_of_contraction_factor(Points<Type1>	*big_list,
											long	left_section_index,
											long	right_section_index,
											long	left_interval_index,
											long	right_interval_index)
{
	long	section_index,interval_index;
	double	a,e,contraction_factor;
	double	arithmitis=0,paronomastis=0;
	double	YI,YF,Yp,Yq,KSIn,Yn,Ym,An,Bn;
	
	a=(right_section_index-left_section_index)/(right_interval_index-left_interval_index);
	e=(right_interval_index*left_section_index-left_interval_index*right_section_index)
		/(right_interval_index-left_interval_index);
	interval_index=right_interval_index;
	big_list->goto_index(left_interval_index);
	YI=big_list->point_value();
	big_list->goto_index(right_interval_index);
	YF=big_list->point_value();
	big_list->goto_index(left_section_index);
	Yp=big_list->point_value();
	big_list->goto_index(right_section_index);
	Yq=big_list->point_value();
	while(interval_index<=right_interval_index)
	{
		KSIn=(right_interval_index-interval_index)/(right_interval_index-left_interval_index);
		big_list->goto_index(interval_index);
		Yn=big_list->point_value();
		An=Yn-(KSIn*YI+(1-KSIn)*YF);
		section_index=round(a*interval_index+e);
		big_list->goto_index(section_index);
		Ym=big_list->point_value();
		Bn=Ym-(KSIn*Yp+(1-KSIn)*Yq);
		arithmitis+=Bn*An;
		paronomastis+=An*An;
		interval_index++;
	}
	if (paronomastis!=0)
		contraction_factor=arithmitis/paronomastis;
	else
		contraction_factor=0.5;
	return contraction_factor;
}


template <class Type1,class Type2>
double		heucledian_distance(Points<Type1> *big_list,
								Points2D<Type2> *small_list,
								long	left, long right)
{
	long	big_list_index=big_list->index_value();
	long	small_list_index=small_list->index_value();
	double	x0,y0,x1,y1,x2,y2,y,HD=0;
	int		i=left+1;
	long	plithos=0;
	
	HD=0;
	small_list->goto_first_point();
	x0=small_list->pointx_value();
	y0=small_list->pointy_value();
	big_list->goto_index(left);
	x1=big_list->index_value();
	y1=big_list->point_value();
	big_list->goto_next_point();
	x2=big_list->index_value();
	y2=big_list->point_value();
	
	int counter=0;
	while(i<=right)
	{
		if (x1==x0)
		{
			HD+=fabs(y1-y0);
			plithos++;
			small_list->goto_next_point();
			x0=small_list->pointx_value();
			y0=small_list->pointy_value();
		}
		else if (x2==x0)
		{
			HD+=fabs(y2-y0);
			plithos++;
			big_list->goto_next_point();
			i++;
			x1=x2;
			y1=y2;
			x2=big_list->index_value();
			y2=big_list->point_value();
			small_list->goto_next_point();
			x0=small_list->pointx_value();
			y0=small_list->pointy_value();
		}
		else if ((x0<x2)&&(x0>x1))
		{
			y=(y2-y1)/(x2-x1)*(x0-x1)+y1;
			HD+=fabs(y0-y);
			plithos++;
			small_list->goto_next_point();
			x0=small_list->pointx_value();
			y0=small_list->pointy_value();
		}
		else if (x0>x2)
		{
			big_list->goto_next_point();
			i++;
			x1=x2;
			y1=y2;
			x2=big_list->index_value();
			y2=big_list->point_value();
		}
	}
	/*END*/
	big_list->goto_index(big_list_index);
	small_list->goto_index(small_list_index);
	return(HD/plithos);
}


template <class Type>
double	LINE_SNR_RATIO(Points<Type> *signal1,Points<Type> *signal2)
{
	long	N=signal1->total_points();
	double	arithmitis=signal1->square_sum()-sqr(signal1->sum())/N;
	double	square_sum=0,sum=0,paronomastis;
	long	i;
	
	signal1->goto_first_point();
	signal2->goto_first_point();
	for (i=1;i<=N;i++)
	{
		square_sum+=sqr(signal1->point_value()-signal2->point_value());
		sum+=signal1->point_value()-signal2->point_value();
		signal1->goto_next_point();
		signal2->goto_next_point();
	}
	paronomastis=square_sum-sqr(sum)/N;
	return(10*log(arithmitis/paronomastis)/log(10.0));
}

//*****************************************************************

#endif