

#include <time.h>
#include <string.h>
#include <stdio.h>
#include <iostream.h>
#include "mem.h"
#include "mypoints.h"
#include "IFS_map.h"
#include "prngs.h""


#include <time.h>
#include <math.h>

#define POINTS_BUFFER	10000000



//Each of the following functions takes an initial set (small_attractor) of points
//(that are equally distributed along the x-axes),
//the parameters of the IFS (or HVIFS) and produces and returns the
//points that will be created after the application of the IFS.
//N: is the number of maps
//the parameters of the IFSs are contained inside matrices (a,e,s etc.).
//interval_length: the distance (in x-axes) between the last
//and the first interpolation point.

/****************************************************************************/

Points<double> *IFSR2_simple_map_points2D(Points<double> *small_attractor,
								   double* a,double* e,double* s,double* c,double*f,
								   uint2 N,double interval_length)
{	//N: the number of sections
	long	pos,total,section;
	double	x,y,wx,wy,y1;
	double	i;

	long	points=small_attractor->total_points()-1;
	total=N*(points)+1;
	Points<double> *mapped_attractor=new Points<double>(total);
	pos=1;
	//for each section
	for (section=1;section<=N;section++){
		//map_each point
		small_attractor->goto_first_point();
		i=0;
		for (pos=1;pos<=points;pos++){ 
			x=interval_length/(points)*i;
			i++;
			y=small_attractor->point_value(pos);
			//y1=small_attractor->point_value(pos-1);
			/*if (y1>0)
				y1=log(y1)/log(1.5);
			else
				y1=-log(-y1)/log(1.5);*/
			//wx=a[section]*x+e[section];
			wy=c[section]*x+s[section]*y+f[section];//+y1;
			mapped_attractor->put_point(wy); 
		}
	}
	//put the last point
	x=interval_length;
	y=small_attractor->point_value(points+1);
	wx=a[N]*x+e[N];
	wy=c[N]*x+s[N]*y+f[N];
	mapped_attractor->put_point(wy);
	return mapped_attractor;
}






/****************************************************************************/

void IFSR2_simple_map_points2D_with_files(FILE	*read_stream, FILE *write_stream,
								   double* a,double* e,double* s,double* c,double*f,
								   uint2 N,double interval_length,double temp_mod)
{	//N: the number of sections
	long	pos,total,section;
	double	x,y,wx,wy,y1;
	double	i;
	Points<double> *small_attractor;
	uint4	points,items_read,items_write,items;
	fpos_t	file_pos;

	fseek(read_stream,0,SEEK_END);
	fgetpos(read_stream,&file_pos);
	
	points=file_pos/sizeof(double)-1;  
	total=N*(points)+1;
	double		*buffer=(double*)malloc(sizeof(double)*POINTS_BUFFER);
	double		*write_buffer=(double*)malloc(sizeof(double)*POINTS_BUFFER);
	pos=1;
	//for each section
	for (section=1;section<=N;section++){
		//start file from the beggining
		fseek(read_stream,0,SEEK_SET);
		fgetpos(read_stream,&file_pos);
		i=0;
		while (file_pos<points){
			items_read=fread(buffer,sizeof(double),POINTS_BUFFER,read_stream);
			if (items_read==POINTS_BUFFER){
				for (pos=1;pos<=items_read;pos++){
					x=interval_length/(points)*i;
					i++;
					y=buffer[pos-1];
					wy=c[section]*x+s[section]*y+f[section];
					write_buffer[pos-1]=myfmod(wy,temp_mod);
				}
				items=items_read;
			}
			else{
				for (pos=1;pos<items_read;pos++){
					x=interval_length/(points)*i;
					i++;
					y=buffer[pos-1];
					wy=c[section]*x+s[section]*y+f[section];
					write_buffer[pos-1]=myfmod(wy,temp_mod);
				}
				items=items_read-1;
			}
			items_write=fwrite(write_buffer,sizeof(double),items,write_stream);
			file_pos+=items_read;
		}
	}
	//put the last point
	x=interval_length;
	y=buffer[items_read-1];
	wx=a[N]*x+e[N];
	wy=c[N]*x+s[N]*y+f[N];
	wy=myfmod(wy,temp_mod);
	items_write=fwrite(&wy,sizeof(double),1,write_stream);
	free(buffer);
	free(write_buffer);
}




/****************************************************************************/

void HVIFSR2_simple_map_points2D_with_files(FILE	*read_stream, FILE *write_stream,
								   double* a,double* e,double* s11,double* s12, double *s21,
								   double *s22,double* c,double *d, double*f,
								   uint2 N,double interval_length,double temp_mod)
{	//N: the number of sections
	long	pos,total,section;
	double	x,y1,y2,wx,wy1,wy2;
	double	i;
	Points<double> *small_attractor;
	uint4	points,items_read,items_write,items;
	fpos_t	file_pos;

	fseek(read_stream,0,SEEK_END);
	fgetpos(read_stream,&file_pos);
	
	points=file_pos/sizeof(double)-1;  
	total=N*(points)+1;
	double		*buffer=(double*)malloc(sizeof(double)*POINTS_BUFFER);
	double		*write_buffer=(double*)malloc(sizeof(double)*POINTS_BUFFER);
	pos=1;
	//for each section
	for (section=1;section<=N;section++){
		//start file from the beggining
		fseek(read_stream,0,SEEK_SET);
		fgetpos(read_stream,&file_pos);
		i=0;
		while (file_pos<points){
			items_read=fread(buffer,sizeof(double),POINTS_BUFFER,read_stream);
			if (items_read==POINTS_BUFFER){
				for (pos=1;pos<=items_read;pos++){
					x=interval_length/(points)*i;
					i++;
					y1=buffer[pos-1];
					y2=buffer[pos];
					wy1=c[section]*x+s11[section]*y1+s12[section]*y2+d[section];
					wy2=e[section]*x+s21[section]*y1+s22[section]*y2+f[section];
					write_buffer[pos-1]=fmod(wy1,temp_mod);
					write_buffer[pos]=fmod(wy2,temp_mod);
					pos++;
				}
				items=items_read;
			}
			else{
				for (pos=1;pos<items_read;pos++){
					x=interval_length/(points)*i;
					i++;
					y1=buffer[pos-1];
					y2=buffer[pos];
					wy1=c[section]*x+s11[section]*y1+s12[section]*y2+d[section];
					wy2=e[section]*x+s21[section]*y1+s22[section]*y2+f[section];
					write_buffer[pos-1]=fmod(wy1,temp_mod);
					write_buffer[pos]=fmod(wy2,temp_mod);
					pos++;
				}
				items=items_read-1;
			}
			items_write=fwrite(write_buffer,sizeof(double),items,write_stream);
			file_pos+=items_read;
		}
	}
	//put the last point
	x=interval_length;
	y1=buffer[pos-1];
	y2=buffer[++pos];
	//wx=a[N]*x+b[N];
	wy1=c[section]*x+s11[section]*y1+s12[section]*y2+d[section];
	wy2=e[section]*x+s21[section]*y1+s22[section]*y2+f[section];
	wy1=fmod(wy1,temp_mod);
	wy2=fmod(wy2,temp_mod);
	items_write=fwrite(&wy1,sizeof(double),1,write_stream);
	items_write=fwrite(&wy2,sizeof(double),1,write_stream);
	free(buffer);
	free(write_buffer);
}




/****************************************************************************/

Points<double> *HVIFSR2_simple_map_points2D(Points<double> *small_attractor,
								   double* a,double* c,double* e,double* s11,double* s12,
								   double* s21,double* s22,double* b,double*d,double*f,
								   uint2 N,double interval_length)
{	//N: the number of sections
	long	pos,total,section;
	double	x,y,wx,wy1,wy2,y1,y2;
	double	i;

	long	points=small_attractor->total_points()/2-1;
	total=2*(N*(points)+1);
	Points<double> *mapped_attractor=new Points<double>(total);
	pos=1;
	//for each section
	for (section=1;section<=N;section++){
		//map_each point
		small_attractor->goto_first_point();
		i=0;
		for (pos=1;pos<=points;pos++){ 
			x=interval_length/(points)*i;
			i++;
			y1=small_attractor->point_value(2*pos-1);
			y2=small_attractor->point_value(2*pos);
			wy1=c[section]*x+s11[section]*y1+s12[section]*y2+d[section];
			wy2=e[section]*x+s21[section]*y1+s22[section]*y2+f[section];
			mapped_attractor->put_point(wy1); 
			mapped_attractor->put_point(wy2);
		}
	}
	//put the last point
	x=interval_length;
	y1=small_attractor->point_value(2*points+1);
	y2=small_attractor->point_value(2*points+2);
	wy1=c[N]*x+s11[N]*y1+s12[N]*y2+d[N];
	wy2=e[N]*x+s21[N]*y1+s22[N]*y2+f[N];
	mapped_attractor->put_point(wy1);
	mapped_attractor->put_point(wy1);
	return mapped_attractor;
}


